Package: startup
Version: 0.23.0
Title: Friendly R Startup Configuration
Depends: R (>= 2.14.0)
Suggests: commonmark, tools, tcltk
VignetteBuilder: startup
Authors@R: c(person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
                                          email = "henrikb@braju.com"))
Description: Adds support for R startup configuration via '.Renviron.d' and '.Rprofile.d' directories in addition to '.Renviron' and '.Rprofile' files.  This makes it possible to keep private / secret environment variables separate from other environment variables.  It also makes it easier to share specific startup settings by simply copying a file to a directory.
License: LGPL (>= 2.1)
LazyLoad: TRUE
URL: https://henrikbengtsson.github.io/startup/,
        https://github.com/HenrikBengtsson/startup
BugReports: https://github.com/HenrikBengtsson/startup/issues
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-12-07 23:40:45 UTC; henrik
Author: Henrik Bengtsson [aut, cre, cph]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2024-12-08 00:00:01 UTC
Built: R 4.6.0; ; 2026-04-27 03:56:20 UTC; unix
