# Generated by roxygen2: do not edit by hand

export(check)
export(current_script)
export(find_renviron)
export(find_renviron_d)
export(find_rprofile)
export(find_rprofile_d)
export(install)
export(is_debug_on)
export(on_session_enter)
export(on_session_exit)
export(renviron_d)
export(reset_when_cache)
export(restart)
export(rprofile_d)
export(startup)
export(startup_session_options)
export(sysinfo)
export(uninstall)
export(warn)
