## ERROR: If your .Rprofile file does not have a newline
## at the last line, then R itself will simply ignore
## the file silently during the startup process.
##
## The startup::startup() function will check your
## .Rprofile files for this mistake and try to fix
## the problem iff possible.
message("This line does not have a newline")