# CRAN submission startup 0.22.0

on 2024-07-29


## Notes not sent to CRAN

### R CMD check validation

The package has been verified using `R CMD check --as-cran` on:

| R version | GitHub | mac/win-builder |
| --------- | ------ | --------------- |
| 3.5.x     | L      |                 |
| 4.2.x     | L      |                 |
| 4.3.x     | L M  W |                 |
| 4.4.x     | L M  W | M1 W            |
| devel     | L M1 W |    W            |

_Legend: OS: L = Linux, M = macOS, M1 = macOS M1, W = Windows_
