% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_jupyter.R
\name{is_jupyter}
\alias{is_jupyter}
\title{Checks if running R via Jupyter}
\usage{
is_jupyter()
}
\value{
A logical
}
\description{
Checks if running R via Jupyter
}
