% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_microsoftr.R
\name{is_microsoftr}
\alias{is_microsoftr}
\title{Check if running R via Microsoft R Open}
\usage{
is_microsoftr()
}
\value{
A logical
}
\description{
Check if running R via Microsoft R Open
}
