% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_ess.R
\name{is_ess}
\alias{is_ess}
\title{Check if running R via Emacs Speaks Statistics (ESS)}
\usage{
is_ess()
}
\value{
A logical
}
\description{
Check if running R via Emacs Speaks Statistics (ESS)
}
