% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sysinfo.R
\name{sysinfo}
\alias{sysinfo}
\title{Information on the current R session}
\usage{
sysinfo()
}
\value{
A named list.
}
\description{
Information on the current R session
}
\examples{
startup::sysinfo()

}
